--- src/global/mail_proto.h.orig	Fri Mar  9 13:34:50 2007
+++ src/global/mail_proto.h	Fri Mar  9 13:35:30 2007
@@ -198,6 +198,7 @@
 #define XCLIENT_ADDR		"ADDR"		/* client address */
 #define XCLIENT_PROTO		"PROTO"		/* client protocol */
 #define XCLIENT_HELO		"HELO"		/* client helo */
+#define XCLIENT_LOGIN		"LOGIN"		/* authenticated client SASL login */
 
 #define XCLIENT_UNAVAILABLE	"[UNAVAILABLE]"	/* permanently unavailable */
 #define XCLIENT_TEMPORARY	"[TEMPUNAVAIL]"	/* temporarily unavailable */
--- src/smtpd/smtpd.c.orig2	Fri Mar  9 13:51:27 2007
+++ src/smtpd/smtpd.c	Fri Mar  9 13:53:34 2007
@@ -1499,10 +1499,11 @@
     /* XCLIENT must not override its own access control. */
     if ((discard_mask & EHLO_MASK_XCLIENT) == 0)
 	if (xclient_allowed)
+	    /* XXX - XCLIENT_LOGIN should be adversed only if(var_smtpd_sasl_enable) */
 	    ENQUEUE_FIX_REPLY(state, reply_buf, XCLIENT_CMD
 			      " " XCLIENT_NAME " " XCLIENT_ADDR
 			      " " XCLIENT_PROTO " " XCLIENT_HELO
-			      " " XCLIENT_REVERSE_NAME);
+			      " " XCLIENT_REVERSE_NAME " " XCLIENT_LOGIN);
     if ((discard_mask & EHLO_MASK_XFORWARD) == 0)
 	if (xforward_allowed)
 	    ENQUEUE_FIX_REPLY(state, reply_buf, XFORWARD_CMD
@@ -3061,6 +3062,9 @@
     };
     int     got_helo = 0;
     int     got_proto = 0;
+#ifdef USE_SASL_AUTH
+    int     got_sasl_login = 0;
+#endif
 
     /*
      * Sanity checks.
@@ -3234,6 +3238,20 @@
 	    got_proto = 1;
 	}
 
+#ifdef USE_SASL_AUTH
+	/*
+	 * LOGIN=substitute SMTP client SASL login. Also set method as XCLIENT
+	 */
+	else if (var_smtpd_sasl_enable && STREQ(attr_name, XCLIENT_LOGIN)) {
+	    /*
+	     * XXX may be here need some saninty check of attr_value
+	     * but we trast to frontend
+	     */
+	    UPDATE_STR(state->sasl_username, attr_value);
+	    UPDATE_STR(state->sasl_method, XCLIENT_CMD);
+	    got_sasl_login = 1;
+	}
+#endif
 	/*
 	 * Unknown attribute name. Complain.
 	 */
@@ -3282,7 +3300,7 @@
 	state->protocol = mystrdup(MAIL_PROTO_SMTP);
     }
 #ifdef USE_SASL_AUTH
-    if (var_smtpd_sasl_enable)
+    if (got_sasl_login == 0 && var_smtpd_sasl_enable)
 	smtpd_sasl_auth_reset(state);
 #endif
     chat_reset(state, 0);
